/*==========================================================================
//  Implementation of a social learning PSO (SL-PSO) for scalable optimization
//
//  See the details of SL-PSO in the following paper
//  R. Cheng and Y. Jin, A Social Learning Particle Swarm Optimization Algorithm for Scalable Pptimization,
//  Information Sicences, 2014
//
//  The source code SL-PSO is implemented by Ran Cheng 
//
//  If you have any questions about the code, please contact: 
//  Ran Cheng at r.cheng@surrey.ac.uk 
//  Prof. Yaochu Jin at yaochu.jin@surrey.ac.uk
===========================================================================*/


#include"stdio.h"
#include"string.h"
#include"time.h"
#include"stdlib.h"
#include"rand.h"
#include"math.h"


#define MAXPOP 1000
#define MAXDIM 5000


//declaration of variables and arrays
int popsize,d, maxgen, boundarycontrol, bestidx, gennum;
int prank[MAXPOP];
double p[MAXPOP][MAXDIM], v[MAXPOP][MAXDIM], fitness[MAXPOP];
double best, bestp[MAXDIM]; 
double center[MAXDIM];
double r1,r2,r3,phi;
double upper,lower;
double seed;
double prob[MAXPOP];
int FEs,funcid, maxrun;
int initialize();

//get ranking of each individual
int getprank(int prank[]);
//get the center (i.e., the mean) of the population
int getcenter();
//social learning 
int learning(int ranki);

//other functions
int input();
int initialize();
int print(int FEs);
int boundary(double a[], int dim);
double F(int funcid, double a[], int dim);
int procedure(int runnum);
int run(int maxrun);
double Sphere(double a[], int dim);




//main function goes here
int main(){
	input();
	run(maxrun);
	return 0;
}

int run(int maxrun){
	
	for(int i = 0; i < maxrun; i++){
		seed = seed/(double)(i + 1);
		procedure(i);
	}
	
	return 0;
}


//main procedure
int procedure(int runnum){
	gennum = 0;
	initialize();
	printf("%Run No.%d\n", runnum);
	//main loop
	while(gennum < maxgen){

		if(gennum % 10 == 0)
			print(gennum);

		getcenter();
		
		getprank(prank);
		
		for(int i = 0; i < popsize - 1; i++){

				if(flip(prob[i]))
					learning(i);
		}

		memcpy(bestp, p[bestidx], sizeof(bestp));
		gennum++;
		
	}
	return 0;
}

int input(){

	//problem dimensionality
	d = 100;
	//problem boundary
	lower = -5.0; upper = 5.0;
	//random seed
	seed = 0.333;
	//number of max runs 
	maxrun = 1;
	//number of max generations in each run
	maxgen = 50*d;
	//population size
	popsize = 100 + (int)((double)d/10.0);
	//social factor 
	phi = (double)d/100.0*0.01;

	return 0;
}

int initialize(){
	randomize(seed);
	for(int i = 0; i < popsize; i++){
		for(int j = 0; j < d; j++)
			{
				p[i][j] = rndreal(lower,upper); 
				v[i][j] = 0;
			}
		prob[i] = 1 - (double)i/(double)(popsize - 1);
		
		prob[i] = pow(prob[i], log( sqrt( ceil( (double)d/100.0 ) ) ));
	}
	best = 1e20;

	for(int i = 0; i < popsize; i++){
		fitness[i] = F(funcid, p[i], d);
		if(fitness[i]  < best){
			best = fitness[i] ;
			bestidx = i;
		}
	}

	FEs = popsize;
	memcpy(bestp, p[bestidx], sizeof(bestp));

	return 0;
}


//get the mean of the population, i.e., the center
int getcenter(){
	for(int j = 0; j < d; j++){
		center[j] = 0;
		for(int i = 0; i < popsize; i++){
			center[j] += p[i][j];
		}
		center[j] /= (double)popsize;
	}
	return 0;
}


//the learning procedure: each individal learns from demonstrators
int learning(int ranki){
	int loseidx, winidx;
	loseidx = prank[ranki]; 
	
	for(int j = 0; j < d; j++){
		int lower = ranki + 1;
		int upper = popsize - 1;
			r1 = randomperc();
			r2 = randomperc();
			r3 = randomperc();
			winidx = prank[rnd(lower, upper)];
			v[loseidx][j] = r1*v[loseidx][j] 
											+ r2*(p[winidx][j] - p[loseidx][j])
											+ r3*phi*(center[j] - p[loseidx][j]);
			p[loseidx][j] += v[loseidx][j];
	}

	
	//boundary control
	if(boundarycontrol)
		boundary(p[loseidx], d); 

	//fitness evaluation
	fitness[loseidx] = F(funcid, p[loseidx], d);
	if(fitness[loseidx]  < best){
		best = fitness[loseidx] ;
		bestidx = loseidx;
	}

	return 0;
}

//get the ranking of the individuals
int getprank(int prank[]){
	double max;
	int used[MAXPOP];
	int k;
	for(int i = 0; i < popsize; i++){
		prank[i] = -1;
		used[i] = 0;
	}
	for(int i = 0; i < popsize; i++){
		max = -1e20;
		k = 0;
		for(int j = 0; j < popsize; j++)
			if(!used[j] && fitness[j] > max){
				k = j;
				max = fitness[j];
			}
		prank[i] = k;
		used[k] = 1;
	}
	return 0;

}



//fitness evaluation
double F(int funcid, double a[], int dim){
	FEs++;
	return Sphere(a, dim);
}


//boundary control
int boundary(double a[], int dim){
	for(int i = 0; i < dim; i++){
		if(a[i] < lower)
			a[i] = lower;
		if(a[i] > upper)
			a[i] = upper;
	}
	return 0;
}


//Sphaere function
double Sphere(double a[], int dim){

	int i;
	double sum=0.0;
	for(i=0; i< dim; i++)
	{
		sum+=(a[i])*(a[i]);
	}
	return sum;
}


int print(int gennum){
	printf("genenration = %d: %.6e\n", gennum, best);
	return 0;
}

